@@ -3,15 +3,22 @@ from __future__ import annotations
 from e3.anod.error import AnodError
 from e3.main import Main
 
+from typing import TYPE_CHECKING
 
-def anod_cmdline(subparsers, action_name, action_help):
+if TYPE_CHECKING:
+    from argparse import _SubParsersAction
+
+
+def anod_cmdline(
+    subparsers: _SubParsersAction, action_name: str, action_help: str
+) -> None:
     parser = subparsers.add_parser(action_name, help=action_help)
     parser.set_defaults(action_name=action_name)
     parser.add_argument("spec", metavar="SPEC", help="The specification file name")
     parser.add_argument("--qualifier")
 
 
-def anod():
+def anod() -> None:
     """bin/anod script entry point.
 
     This script is installed in the sandbox.
@@ -26,12 +33,13 @@ def anod():
     import e3.store
     import e3.store.cache
 
+    assert sys.modules["__main__"].__file__ is not None
+
     sandbox_dir = os.path.abspath(
-        os.path.join(os.path.dirname(sys.modules["__main__"].__file__), os.pardir)
+        os.path.join(os.path.dirname(sys.modules["__main__"].__file__), os.pardir)  # type: ignore
     )
 
-    sandbox = e3.anod.sandbox.SandBox()
-    sandbox.root_dir = sandbox_dir
+    sandbox = e3.anod.sandbox.SandBox(root_dir=sandbox_dir)
 
     # Load the local specs
     spec_repo = e3.anod.loader.AnodSpecRepository(sandbox.specs_dir)
@@ -47,6 +55,7 @@ def anod():
     subparsers = m.argument_parser.add_subparsers()
     anod_cmdline(subparsers, "download", "download a binary package")
     m.parse_args()
+    assert m.args is not None
 
     action = m.args.action_name
     spec = m.args.spec
