@@ -12,7 +12,6 @@ import e3.net.smtp
 from e3.event import EventHandler
 
 if TYPE_CHECKING:
-    from typing import Dict, List, Union
     from e3.event import Event
 
 
@@ -22,7 +21,7 @@ class SMTPHandler(EventHandler):
         subject: str,
         from_addr: str,
         to_addr: str,
-        smtp_servers: Union[str, List[str]],
+        smtp_servers: str | list[str],
     ):
         """Initialize a SMTP event manager."""
         self.default_subject = subject
@@ -34,7 +33,7 @@ class SMTPHandler(EventHandler):
             self.smtp_servers = smtp_servers
 
     @classmethod
-    def decode_config(cls, config_str: str) -> Dict[str, Union[str, List[str]]]:
+    def decode_config(cls, config_str: str) -> dict[str, str | list[str]]:
         subject, from_addr, to_addr, smtp_servers = config_str.split(",", 3)
         return {
             "subject": subject,
@@ -45,7 +44,10 @@ class SMTPHandler(EventHandler):
 
     def encode_config(self) -> str:
         return "{},{},{},{}".format(
-            self.subject, self.from_addr, self.to_addr, ",".join(self.smtp_servers),
+            self.subject,
+            self.from_addr,
+            self.to_addr,
+            ",".join(self.smtp_servers),
         )
 
     @property
@@ -74,7 +76,7 @@ class SMTPHandler(EventHandler):
         event_json.add_header("Content-Disposition", "attachment", filename="event")
         mail.attach(event_json)
 
-        result: Dict[str, Dict[str, dict]] = {"attachments": {}}
+        result: dict[str, dict[str, dict]] = {"attachments": {}}
 
         for name, (filename, _) in list(attachments.items()):
             ctype, encoding = mimetypes.guess_type(filename)
