@@ -1,4 +1,5 @@
 """High-Level file manipulation."""
+
 from __future__ import annotations
 
 import fnmatch
@@ -12,6 +13,9 @@ import stat
 import sys
 from collections import namedtuple
 from typing import TYPE_CHECKING
+from packaging.version import Version
+from pathlib import Path
+from platform import python_version
 
 import e3
 import e3.error
@@ -22,7 +26,7 @@ from e3.collection.trie import Trie
 logger = e3.log.getLogger("fs")
 
 if TYPE_CHECKING:
-    from typing import Iterable, Optional
+    from typing import Iterable, Any
     from collections.abc import Callable, Sequence
 
 
@@ -31,8 +35,8 @@ class FSError(e3.error.E3Error):
 
 
 def cp(
-    source: str,
-    target: str,
+    source: str | Path,
+    target: str | Path,
     copy_attrs: bool = True,
     recursive: bool = False,
     preserve_symlinks: bool = False,
@@ -61,7 +65,11 @@ def cp(
         logger.warning("recursive copy always preserves file attributes")
 
     # Compute file list and number of file to copy
-    file_list = ls(source, emit_log_record=False)
+    file_list: list[str] | list[Path]
+    if isinstance(source, Path):
+        file_list = [source]
+    else:
+        file_list = ls(source, emit_log_record=False)
     file_number = len(file_list)
 
     if file_number == 0:
@@ -78,7 +86,7 @@ def cp(
             if os.path.isdir(target):
                 f_dest = os.path.join(target, os.path.basename(f))
             else:
-                f_dest = target
+                f_dest = os.fspath(target)
 
             if recursive and os.path.isdir(f):
                 shutil.copytree(f, f_dest, symlinks=preserve_symlinks)
@@ -97,7 +105,7 @@ def cp(
 
 
 def directory_content(
-    path: str, include_root_dir: bool = False, unixpath: bool = False
+    path: str | Path, include_root_dir: bool = False, unixpath: bool = False
 ) -> list[str]:
     """Return the complete directory content (recusrsively).
 
@@ -117,9 +125,11 @@ def directory_content(
             result.append(os.path.join(root, d) + os.sep)
     if not include_root_dir:
         result = [
-            os.path.relpath(e, path) + os.sep
-            if e.endswith(os.sep)
-            else os.path.relpath(e, path)
+            (
+                os.path.relpath(e, path) + os.sep
+                if e.endswith(os.sep)
+                else os.path.relpath(e, path)
+            )
             for e in result
         ]
     if unixpath:
@@ -128,7 +138,9 @@ def directory_content(
     return result
 
 
-def echo_to_file(filename: str, content: str | list[str], append: bool = False) -> None:
+def echo_to_file(
+    filename: str | Path, content: str | list[str], append: bool = False
+) -> None:
     """Output content into a file.
 
     This function is useful when writing few content to a file for which we
@@ -151,8 +163,8 @@ def echo_to_file(filename: str, content: str | list[str], append: bool = False)
 
 
 def find(
-    root: str,
-    pattern: Optional[str] = None,
+    root: str | Path,
+    pattern: str | None = None,
     include_dirs: bool = False,
     include_files: bool = True,
     follow_symlinks: bool = False,
@@ -182,16 +194,21 @@ def find(
     return result
 
 
-def get_filetree_state(path: str, ignore_hidden: bool = True) -> str:
+def get_filetree_state(
+    path: str | Path, ignore_hidden: bool = True, hash_content: bool = False
+) -> str:
     """Compute a hash on a filetree to reflect its current state.
 
     :param path: root path of the file tree to be checked
     :param ignore_hidden: if True (default) then files and directories
         tarting with a dot are ignored.
+    :param hash_content: if True, include the content in the hash.
+
     :return: a hash as a string
 
-    The function will not report changes in the hash if a file is modified
-    and its attributes (size, modification time and mode) are not changed.
+    By default, the function will not report changes in the hash if a file is
+    modified and its attributes (size, modification time and mode) are not
+    changed.
     This case is quite uncommon. By ignoring it we can compute efficiently a
     hash representing the state of the file tree without having to read the
     content of all files.
@@ -205,8 +222,12 @@ def get_filetree_state(path: str, ignore_hidden: bool = True) -> str:
         )
         return state.encode("utf-8")
 
+    def get_content(file_path: str) -> bytes:
+        with open(file_path, "rb") as f:
+            return f.read()
+
     path = os.path.abspath(path)
-    result = hashlib.sha1()
+    result = hashlib.sha1()  # nosec
     if os.path.isdir(path):
         for root, dirs, files in os.walk(path):
             if ignore_hidden:
@@ -225,12 +246,21 @@ def get_filetree_state(path: str, ignore_hidden: bool = True) -> str:
                 full_path = os.path.join(root, path)
                 result.update(compute_state(full_path))
 
+                if hash_content:
+                    result.update(get_content(full_path))
+
     else:
         result.update(compute_state(path))
+
+        if hash_content:
+            result.update(get_content(path))
+
     return result.hexdigest()
 
 
-def ls(path: str | list[str], emit_log_record: bool = True) -> list[str]:
+def ls(
+    path: str | Iterable[str] | Path | Iterable[Path], emit_log_record: bool = True
+) -> list[str]:
     """list files.
 
     :param path: glob pattern or glob pattern list
@@ -241,10 +271,10 @@ def ls(path: str | list[str], emit_log_record: bool = True) -> list[str]:
     This function do not raise an error if no file matching the glob pattern
     is encountered. The only consequence is that an empty list is returned.
     """
-    if isinstance(path, str):
-        path_list = [path]
+    if isinstance(path, (str, Path)):
+        path_list = [os.fspath(path)]
     else:
-        path_list = list(path)
+        path_list = [os.fspath(p) for p in path]
 
     if emit_log_record:
         logger.debug("ls %s", " ".join(path_list))
@@ -252,7 +282,7 @@ def ls(path: str | list[str], emit_log_record: bool = True) -> list[str]:
     return sorted(itertools.chain.from_iterable(glob.glob(p) for p in path_list))
 
 
-def mkdir(path: str, mode: int = 0o755, quiet: bool = False) -> None:
+def mkdir(path: str | Path, mode: int = 0o755, quiet: bool = False) -> None:
     """Create a directory.
 
     :param path: path to create. If intermediate directories do not exist
@@ -282,10 +312,11 @@ def mkdir(path: str, mode: int = 0o755, quiet: bool = False) -> None:
             raise FSError(origin="mkdir", message=f"can't create {path}") from e
 
 
-def mv(source: str | list[str], target: str) -> None:
+def mv(source: str | Path | Iterable[str] | Iterable[Path], target: str | Path) -> None:
     """Move files.
 
-    :param source: a glob pattern
+    :param source: a glob pattern, a sequence/iterator of glob patterns, a path,
+        or a sequence/iterator of paths
     :param target: target file or directory. If the source resolves as
         several files then target should be a directory
 
@@ -300,6 +331,7 @@ def mv(source: str | list[str], target: str) -> None:
         rmtree. This ensure moving a directory with read-only files will
         work.
         """
+        logger.debug(f"mv {src} {dst}")
 
         def same_file(src: str, dst: str) -> bool:
             if hasattr(os.path, "samefile"):
@@ -357,11 +389,6 @@ def mv(source: str | list[str], target: str) -> None:
                 rm(src)
         return
 
-    if isinstance(source, str):
-        logger.debug("mv %s %s", source, target)
-    else:
-        logger.debug("mv %s %s", " ".join(source), target)
-
     try:
         # Compute file list and number of file to copy
         file_list = ls(source, emit_log_record=False)
@@ -374,24 +401,28 @@ def mv(source: str | list[str], target: str) -> None:
             if os.path.isdir(source) and os.path.isdir(target):
                 move_file(source, os.path.join(target, os.path.basename(source)))
             else:
-                move_file(source, target)
+                move_file(source, os.fspath(target))
         elif not os.path.isdir(target):
             # More than one file to move but the target is not a directory
             raise FSError("mv", f"{target} should be a directory")
         else:
             for f in file_list:
                 f_dest = os.path.join(target, os.path.basename(f))
-                e3.log.debug("mv %s %s", f, f_dest)
                 move_file(f, f_dest)
     except Exception as e:
         logger.error(e)
         raise FSError(origin="mv", message=str(e)) from e
 
 
-def rm(path: str | list[str], recursive: bool = False, glob: bool = True) -> None:
+def rm(
+    path: str | Path | Iterable[str] | Iterable[Path],
+    recursive: bool = False,
+    glob: bool = True,
+) -> None:
     """Remove files.
 
-    :param path: a glob pattern, or a list of glob patterns
+    :param path: a glob pattern, a sequence/iterator of glob patterns, a path,
+        or a sequence/iterator of paths
     :param recursive: if True do a recursive deletion. Default is False
     :param glob: if True globbing pattern expansion is used
 
@@ -400,31 +431,29 @@ def rm(path: str | list[str], recursive: bool = False, glob: bool = True) -> Non
     Note that the function will not raise an error is there are no file to
     delete.
     """
-    if recursive:
-        logger.debug("rm -r %s", str(path))
-    else:
-        logger.debug("rm %s", str(path))
-
     # We transform the list into a set in order to remove duplicate files in
     # the list
     if glob:
         file_list = set(ls(path, emit_log_record=False))
     else:
-        if isinstance(path, str):
-            file_list = {path}
+        if isinstance(path, (str, Path)):
+            file_list = {os.fspath(path)}
         else:
-            file_list = set(path)
+            file_list = {os.fspath(p) for p in path}
 
-    def onerror(func: Callable, error_path: str, exc_info: tuple) -> None:
+    logger.debug(f"rm{' -r' if recursive else ''} {' '.join(file_list)}")
+
+    def onerror(
+        func: Callable[..., Any], error_path: str, exc_info: tuple | BaseException
+    ) -> None:
         """When shutil.rmtree fail, try again to delete the file.
 
         :param func: function to call on error
         :param error_path: file or directory to remove
-        :param exc_info: exception raised when the first delete attempt was
-             made
+        :param exc_info: exception raised when the first delete attempt was made
         """
         del exc_info
-        e3.log.debug("error when running %s on %s", func, error_path)
+        e3.log.debug(f"error when running {func.__name__!r} on {error_path}")
 
         # First check whether the file we are trying to delete exist. If not
         # the work is already done, no need to continue trying removing it.
@@ -460,7 +489,13 @@ def rm(path: str | list[str], recursive: bool = False, glob: bool = True) -> Non
             os.chmod(error_path, 0o700)
 
             # And continue to delete the subdir
-            shutil.rmtree(error_path, onerror=onerror)
+            if Version(python_version()) >= Version("3.12"):
+                shutil.rmtree(error_path, onexc=onerror)
+            else:
+                shutil.rmtree(error_path, onerror=onerror)
+
+        else:
+            raise FSError(origin="rm", message=f"unknown function: {func.__name__!r}")
 
     for f in file_list:
         try:
@@ -476,7 +511,10 @@ def rm(path: str | list[str], recursive: bool = False, glob: bool = True) -> Non
             # Note: shutil.rmtree requires its argument to be an actual
             # directory, not a symbolic link to a directory
             if recursive and os.path.isdir(f) and not os.path.islink(f):
-                shutil.rmtree(f, onerror=onerror)
+                if Version(python_version()) >= Version("3.12"):
+                    shutil.rmtree(f, onexc=onerror)
+                else:
+                    shutil.rmtree(f, onerror=onerror)
             else:
                 e3.os.fs.force_remove_file(f)
 
@@ -487,13 +525,13 @@ def rm(path: str | list[str], recursive: bool = False, glob: bool = True) -> Non
             ) from e
 
 
-def splitall(path: str) -> tuple[str, ...]:
+def splitall(path: str | Path) -> tuple[str, ...]:
     """Split a path into a list of path components.
 
     :param path: path to split
     :return: a list of path components
     """
-    dirnames = []  # type: list[str]
+    dirnames: list[str] = []
     while 1:
         head, tail = os.path.split(path)
         if head == path:
@@ -533,14 +571,20 @@ VCS_IGNORE_LIST = (
     ".gitreview",
     ".mailmap",
     ".idea",
+    ".python-version",
+    ".gitlab-ci",
+    ".gitlab-ci.yml",
+    ".vscode",
+    ".github",
+    "comment",
 )
 
 
 def sync_tree(
-    source: str,
-    target: str,
-    ignore: Optional[str | Sequence[str]] = None,
-    file_list: Optional[list[str]] = None,
+    source: str | Path,
+    target: str | Path,
+    ignore: str | Sequence[str] | None = None,
+    file_list: list[str] | None = None,
     delete: bool = True,
     preserve_timestamps: bool = True,
     delete_ignore: bool = False,
@@ -602,7 +646,7 @@ def sync_tree(
         ignore_path_prefixes = Trie(match_delimiter="/")
 
         ignore_base_regexp_list = []
-        ignore_base_regexp: Optional[re.Pattern[str]] = None
+        ignore_base_regexp: re.Pattern[str] | None = None
 
         for pattern in norm_ignore_list:
             pk = path_key(pattern)
@@ -730,8 +774,9 @@ def sync_tree(
         :param src: the source FileInfo object
         :param dst: the target FileInfo object
         """
+        mode = stat.S_IMODE(src.stat.st_mode)
+
         if islink(src):  # windows: no cover
-            mode = stat.S_IMODE(src.stat.st_mode)
             if hasattr(os, "lchmod"):
                 getattr(os, "lchmod")(dst.path, mode)  # noqa: B009
 
@@ -746,11 +791,12 @@ def sync_tree(
                         or why.errno != errno.EOPNOTSUPP
                     ):
                         raise
+
+                    logger.debug("lchflags: operation not supported [EOPNOTSUPP]")
         else:
-            mode = stat.S_IMODE(src.stat.st_mode)
             if hasattr(os, "utime"):
                 if preserve_timestamps:
-                    os.utime(dst.path, (src.stat.st_atime, src.stat.st_mtime))
+                    os.utime(dst.path, ns=(src.stat.st_atime_ns, src.stat.st_mtime_ns))
                 else:
                     os.utime(dst.path, None)
             if hasattr(os, "chmod"):
@@ -767,7 +813,9 @@ def sync_tree(
                     ):
                         raise
 
-    def safe_copy(src: FileInfo, dst: FileInfo) -> None:
+                    logger.debug("chflags: operation not supported [EOPNOTSUPP]")
+
+    def safe_copy(src: FileInfo, dst: FileInfo, is_directory: bool = False) -> None:
         """Copy src file into dst preserving all attributes.
 
         :param src: the source FileInfo object
@@ -778,7 +826,7 @@ def sync_tree(
             if not islink(dst) or os.readlink(dst.path) != linkto:
                 if dst.stat is not None:
                     rm(dst.path, recursive=True, glob=False)
-                os.symlink(linkto, dst.path)
+                os.symlink(linkto, dst.path, target_is_directory=is_directory)
             copystat(src, dst)
         else:
             if isdir(dst):
@@ -787,12 +835,17 @@ def sync_tree(
                 rm(dst.path, recursive=True, glob=False)
             elif islink(dst):
                 # dst symlink will be replaced by a file having the same
-                #  content as 'src'
+                # content as 'src'
                 rm(dst.path, recursive=False, glob=False)
 
             try:
                 if dst.basename != src.basename:
-                    rm(dst.path, glob=False)
+                    if dst.stat is not None:
+                        # Case in which the destination file exists but does
+                        # not have the same casing. In that case we delete the
+                        # target file and redo a copy. This occurs for example
+                        # on Windos with NTFS.
+                        rm(dst.path, glob=False)
                     dst = FileInfo(
                         os.path.join(os.path.dirname(dst.path), src.basename),
                         None,
@@ -803,7 +856,8 @@ def sync_tree(
                     with open(dst.path, "wb") as fdst:
                         shutil.copyfileobj(fsrc, fdst)
             except OSError:
-                rm(dst.path, glob=False)
+                if dst.stat is not None:
+                    rm(dst.path, glob=False)
                 with open(src.path, "rb") as fsrc:
                     with open(dst.path, "wb") as fdst:
                         shutil.copyfileobj(fsrc, fdst)
@@ -845,7 +899,7 @@ def sync_tree(
                 os.makedirs(dest_dir)
 
     def walk(
-        root_dir: str, target_root_dir: str, entry: Optional[FilesInfo] = None
+        root_dir: str, target_root_dir: str, entry: FilesInfo | None = None
     ) -> Iterable[FilesInfo]:
         """Walk through source and target file trees.
 
@@ -964,7 +1018,9 @@ def sync_tree(
             # the source tree.
             if need_update(wf.source, wf.target):
                 if isfile(wf.source) or islink(wf.source):
-                    safe_copy(wf.source, wf.target)
+                    safe_copy(
+                        wf.source, wf.target, is_directory=os.path.isdir(wf.source.path)
+                    )
                     updated_list.append(wf.target.path)
                 elif isdir(wf.source):
                     safe_mkdir(wf.source, wf.target)
@@ -978,7 +1034,7 @@ def sync_tree(
     return updated_list, deleted_list
 
 
-def extension(path: str) -> str:
+def extension(path: str | Path) -> str:
     """Return the extension of a given filename.
 
     Contrary to os.path.splitext which returns .gz, the function will return
