@@ -1,18 +1,27 @@
 from __future__ import annotations
 
+from typing import TYPE_CHECKING
+
 import hashlib
 import os
 
 import e3.error
 
+if TYPE_CHECKING:
+    from typing import Literal
+    from os import PathLike
+
 
 class HashError(e3.error.E3Error):
     pass
 
 
-def __compute_hash(path: str, kind: str) -> str:
+def __compute_hash(
+    path: PathLike[str] | str,
+    kind: Literal["md5"] | Literal["sha1"] | Literal["sha256"],
+) -> str:
     if not os.path.isfile(path):
-        raise HashError(kind, "cannot find %s" % path)
+        raise HashError(kind, f"cannot find {path}")
 
     with open(path, "rb") as f:
         result = getattr(hashlib, kind)()
@@ -24,7 +33,7 @@ def __compute_hash(path: str, kind: str) -> str:
     return result.hexdigest()
 
 
-def md5(path: str) -> str:
+def md5(path: PathLike[str] | str) -> str:
     """Compute md5 hexadecimal digest of a file.
 
     :param path: path to a file
@@ -35,7 +44,7 @@ def md5(path: str) -> str:
     return __compute_hash(path, "md5")
 
 
-def sha1(path: str) -> str:
+def sha1(path: PathLike[str] | str) -> str:
     """Compute sha1 hexadecimal digest of a file.
 
     :param str path: path to a file
@@ -46,7 +55,7 @@ def sha1(path: str) -> str:
     return __compute_hash(path, "sha1")
 
 
-def sha256(path: str) -> str:
+def sha256(path: PathLike[str] | str) -> str:
     """Compute sha256 hexadecimal digest of a file.
 
     :param str path: path to a file
