@@ -19,6 +19,8 @@ from e3.os.windows.native_api import (
     Access,
     FileAttribute,
     FileInfo,
+    FindData,
+    IOReparseTag,
     IOStatusBlock,
     NTException,
     ObjectAttributes,
@@ -31,6 +33,7 @@ from e3.os.windows.native_api import (
 if TYPE_CHECKING:
     from typing import Any
     from collections.abc import Callable
+    from pathlib import Path
 
 
 logger = e3.log.getLogger("os.windows.fs")
@@ -74,7 +77,7 @@ class NTFile:
     :ivar basic_info: ObjectAttributes object associated with the file
     """
 
-    def __init__(self, filename: str, parent: NTFile | None = None):
+    def __init__(self, filename: str | Path, parent: NTFile | None = None):
         """Initialize a NTFile object.
 
         :param filename: path to the file or basename if parent is not None
@@ -86,7 +89,8 @@ class NTFile:
             self.nt_filename = UnicodeString(f"\\??\\{self.path}")
             self.parent_handle = None
         else:
-            assert "\\" not in filename and "/" not in filename
+            file_path = os.fspath(filename)
+            assert "\\" not in file_path and "/" not in file_path
             self.path = os.path.join(parent.path, filename)
             self.nt_filename = UnicodeString(str(filename))
             self.parent_handle = parent.handle
@@ -215,6 +219,30 @@ class NTFile:
 
         return result.index_number
 
+    @property
+    def reparse_tag(self) -> int:
+        """Find the reparse point tag for a given file.
+
+        :return: the tag as int. 0 is returned if not a reparse point
+        """
+        if not self.is_reparse_point:
+            return 0
+
+        result = FindData()
+        find_first_file: Callable = NT.FindFirstFile  # type: ignore
+        find_close: Callable = NT.FindClose  # type: ignore
+
+        handle = find_first_file(c_wchar_p(self.path), pointer(result))
+        if not handle:
+            raise NTException(
+                status=handle,
+                message=f"cannot find volume for {self.path}",
+                origin="NTFile.reparse_tag",
+            )
+        find_close(handle)
+
+        return result.reserved0
+
     def read_attributes_internal(self) -> None:
         """Retrieve file basic attributes (internal function).
 
@@ -303,6 +331,22 @@ class NTFile:
         """
         return self.basic_info.file_attributes.attr & FileAttribute.READONLY > 0
 
+    @property
+    def is_reparse_point(self) -> bool:
+        """Check if a given file is a reparse point.
+
+        :return: True if the file is a reparse point, False otherwise
+        """
+        return self.basic_info.file_attributes.attr & FileAttribute.REPARSE_POINT > 0
+
+    @property
+    def is_symlink(self) -> bool:
+        """Check whether a given file is a symlink or not.
+
+        :return: return True for all kind of symlinks (native and WSL).
+        """
+        return self.reparse_tag in (IOReparseTag.SYMLINK, IOReparseTag.WSL_SYMLINK)
+
     @property
     def trash_path(self) -> str:
         """Return path in which the file can move safely for deletion.
@@ -501,6 +545,10 @@ class NTFile:
             self.basic_info.file_attributes.attr &= ~FileAttribute.READONLY
             self.write_attributes()
 
+        if self.is_symlink:
+            # If this is a symlink ensure that we don't delete the target
+            open_options |= OpenOptions.OPEN_REPARSE_POINT
+
         # set our access modes
         desired_access = Access.DELETE
         shared_access = Share.DELETE
