@@ -1,8 +1,9 @@
-"""Generate a SPDX file.
+"""Generate an SPDX file.
 
 This is following the specification from https://spdx.github.io/spdx-spec/v2.3/
 a simple example can be found at ./tests/tests_e3/spdx_test.py
 """
+
 from __future__ import annotations
 
 from enum import Enum, auto
@@ -19,7 +20,14 @@ if TYPE_CHECKING:
     from typing import Literal, Union, Any
 
 NOASSERTION: Literal["NOASSERTION"] = "NOASSERTION"
+"""Indicates that the preparer of the SPDX document is not making any assertion
+regarding the value of this field.
+"""
 NONE_VALUE: Literal["NONE"] = "NONE"
+"""When this value is used as the object of a property it indicates that the
+preparer of the SpdxDocument believes that there is no value for the property.
+This value should only be used if there is sufficient evidence to support this
+assertion."""
 
 if TYPE_CHECKING:
     MAYBE_STR = Union[str, Literal["NOASSERTION"], Literal["NONE"]]
@@ -30,9 +38,25 @@ SPDXID_R = re.compile("[^a-zA-Z0-9.-]")
 class InvalidSPDX(Exception):
     """Raise an exception when the SPDX document cannot be generated."""
 
+    pass
+
+
+class SPDXPackageSupplier(Enum):
+    """Used by the SPDX originator field.
+
+    This field is composed of a package supplier type (organization, person, tool)
+    and a name.
+
+    This enum represents the package supplier type.
+    """
+
+    ORGANIZATION = "Organization"
+    PERSON = "Person"
+    TOOL = "Tool"
+
 
 class SPDXEntry(metaclass=ABCMeta):
-    """Describe a SPDX Entry."""
+    """Describe an SPDX Entry."""
 
     @property
     def entry_key(self) -> str:
@@ -62,7 +86,7 @@ class SPDXEntry(metaclass=ABCMeta):
 
 
 class SPDXEntryStr(SPDXEntry):
-    """Describe a SPDX Entry accepting a string."""
+    """Describe an SPDX Entry accepting a string."""
 
     def __init__(self, value: str) -> None:
         self.value = value
@@ -70,6 +94,11 @@ class SPDXEntryStr(SPDXEntry):
     def __str__(self) -> str:
         return self.value
 
+    def __gt__(self, other: object) -> bool:
+        if isinstance(other, self.__class__):
+            return self.value > other.value
+        return False
+
     def to_json_dict(self) -> dict[str, Any]:
         # Force calling __str__ method to simplify overloading
         # e.g. for SPDXID
@@ -77,7 +106,7 @@ class SPDXEntryStr(SPDXEntry):
 
 
 class SPDXEntryMaybeStr(SPDXEntry):
-    """Describe a SPDX Entry accepting a string, NOASSERTION, or NONE."""
+    """Describe an SPDX Entry accepting a string, NOASSERTION, or NONE."""
 
     def __init__(self, value: MAYBE_STR) -> None:
         self.value = value
@@ -102,7 +131,7 @@ class SPDXEntryMaybeStrMultilines(SPDXEntryMaybeStr):
 
 
 class SPDXEntryBool(SPDXEntry):
-    """Describe a SPDX Entry accepting a boolean."""
+    """Describe an SPDX Entry accepting a boolean."""
 
     def __init__(self, value: bool) -> None:
         self.value: bool = value
@@ -114,11 +143,12 @@ class SPDXEntryBool(SPDXEntry):
         return {self.json_entry_key: self.value}
 
 
+@dataclass
 class SPDXSection:
-    """Describe a SPDX section."""
+    """Describe an SPDX section."""
 
     def to_tagvalue(self) -> list[str]:
-        """Generate a chunk of a SPDX tag:value document.
+        """Generate a chunk of an SPDX tag:value document.
 
         Return a list of SPDX lines
         """
@@ -156,7 +186,8 @@ class SPDXSection:
 class SPDXVersion(SPDXEntryStr):
     """Provide the SPDX version used to generate the document.
 
-    See 6.1 SPDX version field
+    See 6.1 `SPDX version field
+    <https://spdx.github.io/spdx-spec/v2.3/document-creation-information/#61-spdx-version-field>`_.
     """
 
     json_entry_key = "spdxVersion"
@@ -165,14 +196,19 @@ class SPDXVersion(SPDXEntryStr):
 class DataLicense(SPDXEntryStr):
     """License of the SPDX Metadata.
 
-    See 6.2 Data license field
+    See 6.2 `Data license field
+    <https://spdx.github.io/spdx-spec/v2.3/document-creation-information/#62-data-license-field>`_.
     """
 
 
 class SPDXID(SPDXEntryStr):
-    """Identify a SPDX Document, or Package.
+    """Identify an SPDX Document, or Package.
+
+    See 6.3 `SPDX identifier field
+    <https://spdx.github.io/spdx-spec/v2.3/document-creation-information/#63-spdx-identifier-field>`_
+    and 7.2 `Package SPDX identifier field
+    <https://spdx.github.io/spdx-spec/v2.3/package-information/#72-package-spdx-identifier-field>`_.
 
-    See 6.3 SPDX identifier field and 7.2 Package SPDX identifier field
     The value is a unique string containing letters, numbers, ., and/or -.
     """
 
@@ -182,7 +218,7 @@ class SPDXID(SPDXEntryStr):
         # The format of the SPDXID should be "SPDXRef-"[idstring]
         # where [idstring] is a unique string containing letters, numbers, .,
         # and/or -.
-        self.value = re.sub(SPDXID_R, "", value)
+        super().__init__(re.sub(SPDXID_R, "", value))
 
     def __str__(self) -> str:
         return f"SPDXRef-{self.value}"
@@ -197,7 +233,8 @@ class SPDXID(SPDXEntryStr):
 class DocumentName(SPDXEntryStr):
     """Identify name of this document.
 
-    See 6.4 Document name field
+    See 6.4 `Document name field
+    <https://spdx.github.io/spdx-spec/v2.3/document-creation-information/#64-document-name-field>`_.
     """
 
     json_entry_key = "name"
@@ -206,14 +243,16 @@ class DocumentName(SPDXEntryStr):
 class DocumentNamespace(SPDXEntryStr):
     """Provide a unique URI for this document.
 
-    See 6.5 SPDX document namespace field
+    See 6.5 `SPDX document namespace field
+    <https://spdx.github.io/spdx-spec/v2.3/document-creation-information/#65-spdx-document-namespace-field>`_.
     """
 
 
 class LicenseListVersion(SPDXEntryStr):
     """Provide the version of the SPDX License List used.
 
-    See 6.7 License list version field
+    See 6.7 `License list version field
+    <https://spdx.github.io/spdx-spec/v2.3/document-creation-information/#67-license-list-version-field>`_.
     """
 
 
@@ -256,7 +295,8 @@ class EntityRef(SPDXEntry):
 class Creator(EntityRef):
     """Identify who (or what, in the case of a tool) created the SPDX document.
 
-    See 6.8 Creator field
+    See 6.8 `Creator field
+    <https://spdx.github.io/spdx-spec/v2.3/document-creation-information/#68-creator-field>`_.
     """
 
     json_entry_key = "creators"
@@ -265,7 +305,8 @@ class Creator(EntityRef):
 class Created(SPDXEntryStr):
     """Identify when the SPDX document was originally created.
 
-    See 6.9 Created field
+    See 6.9 `Created field
+    <https://spdx.github.io/spdx-spec/v2.3/document-creation-information/#69-created-field>`_.
     """
 
 
@@ -284,7 +325,8 @@ class Tool(Entity):
 class PackageName(SPDXEntryStr):
     """Identify the full name of the package.
 
-    See 7.1 Package name field
+    See 7.1 `Package name field
+    <https://spdx.github.io/spdx-spec/v2.3/package-information/#71-package-name-field>`_
     """
 
     json_entry_key = "name"
@@ -293,7 +335,8 @@ class PackageName(SPDXEntryStr):
 class PackageVersion(SPDXEntryStr):
     """Identify the version of the package.
 
-    See 7.3 Package version field
+    See 7.3 `Package version field
+    <https://spdx.github.io/spdx-spec/v2.3/package-information/#73-package-version-field>`_
     """
 
     json_entry_key = "versionInfo"
@@ -302,14 +345,16 @@ class PackageVersion(SPDXEntryStr):
 class PackageFileName(SPDXEntryStr):
     """Provide the actual file name of the package.
 
-    See 7.4 Package file name field
+    See 7.4 `Package file name field
+    <https://spdx.github.io/spdx-spec/v2.3/package-information/#74-package-file-name-field>`_
     """
 
 
 class PackageSupplier(EntityRef):
     """Identify the actual distribution source for the package.
 
-    See 7.5 Package supplier field
+    See 7.5 `Package supplier field
+    <https://spdx.github.io/spdx-spec/v2.3/package-information/#75-package-supplier-field>`_
     """
 
     json_entry_key = "supplier"
@@ -318,7 +363,8 @@ class PackageSupplier(EntityRef):
 class PackageOriginator(EntityRef):
     """Identify from where the package originally came.
 
-    See 7.6 Package originator field
+    See 7.6 `Package originator field
+    <https://spdx.github.io/spdx-spec/v2.3/package-information/#76-package-originator-field>`_
     """
 
     json_entry_key = "originator"
@@ -327,7 +373,8 @@ class PackageOriginator(EntityRef):
 class PackageDownloadLocation(SPDXEntryMaybeStr):
     """Identifies the download location of the package.
 
-    See 7.7 Package download location field
+    See 7.7 `Package download location field
+    <https://spdx.github.io/spdx-spec/v2.3/package-information/#77-package-download-location-field>`_
     """
 
     json_entry_key = "downloadLocation"
@@ -336,14 +383,16 @@ class PackageDownloadLocation(SPDXEntryMaybeStr):
 class FilesAnalyzed(SPDXEntryBool):
     """Indicates whether the file content of this package have been analyzed.
 
-    See 7.8 Files analyzed field
+    See 7.8 `Files analyzed field
+    <https://spdx.github.io/spdx-spec/v2.3/package-information/#78-files-analyzed-field>`_
     """
 
 
 class PackageChecksum(SPDXEntryStr, metaclass=ABCMeta):
     """Provide a mechanism that permits unique identification of the package.
 
-    See 7.10 Package checksum field
+    See 7.10 `Package checksum field
+    <https://spdx.github.io/spdx-spec/v2.3/package-information/#710-package-checksum-field>`_
     """
 
     entry_key = "PackageChecksum"
@@ -366,6 +415,16 @@ class PackageChecksum(SPDXEntryStr, metaclass=ABCMeta):
         }
 
 
+class PackageHomePage(SPDXEntryMaybeStr):
+    """Identifies the homepage location of the package.
+
+    See 7.11 `Package home page field
+    <https://spdx.github.io/spdx-spec/v2.3/package-information/#711-package-home-page-field>`_
+    """
+
+    json_entry_key = "homepage"
+
+
 class SHA1(PackageChecksum):
     algorithm = "SHA1"
 
@@ -377,7 +436,8 @@ class SHA256(PackageChecksum):
 class PackageLicenseConcluded(SPDXEntryMaybeStr):
     """Contain the license concluded as governing the package.
 
-    See 7.13 Concluded license field
+    See 7.13 `Concluded license field
+    <https://spdx.github.io/spdx-spec/v2.3/package-information/#713-concluded-license-field>`_
     """
 
     json_entry_key = "licenseConcluded"
@@ -386,21 +446,43 @@ class PackageLicenseConcluded(SPDXEntryMaybeStr):
 class PackageLicenseDeclared(SPDXEntryMaybeStr):
     """Contain the license having been declared by the authors of the package.
 
-    See 7.15 Declared license field
+    See 7.15 `Declared license field
+    <https://spdx.github.io/spdx-spec/v2.3/package-information/#715-declared-license-field>`_
     """
 
     json_entry_key = "licenseDeclared"
 
 
+class PackageLicenseComments(SPDXEntryMaybeStrMultilines):
+    """Record background information or analysis for the Concluded License.
+
+    See 7.16 `Comments on license field
+    <https://spdx.github.io/spdx-spec/v2.3/package-information/#716-comments-on-license-field>`_
+    """
+
+    json_entry_key = "licenseComments"
+
+
 class PackageCopyrightText(SPDXEntryMaybeStrMultilines):
     """Identify the copyright holders of the package.
 
-    See 7.17 Copyright text field
+    See 7.17 `Copyright text field
+    <https://spdx.github.io/spdx-spec/v2.3/package-information/#717-copyright-text-field>`_
     """
 
     json_entry_key = "copyrightText"
 
 
+class PackageComment(SPDXEntryMaybeStrMultilines):
+    """Record background information or analysis for the Concluded License.
+
+    See 7.20 `Package comment field
+    <https://spdx.github.io/spdx-spec/v2.3/package-information/#720-package-comment-field>`_
+    """
+
+    json_entry_key = "comment"
+
+
 class ExternalRefCategory(Enum):
     """Identify the category of an ExternalRef."""
 
@@ -410,13 +492,38 @@ class ExternalRefCategory(Enum):
     other = "OTHER"
 
 
+# Create some constants to make writing easier
+SECURITY = ExternalRefCategory.security
+PACKAGE_MANAGER = ExternalRefCategory.package_manager
+PERSISTENT_ID = ExternalRefCategory.persistent_id
+OTHER = ExternalRefCategory.other
+
+# List of valid external reference types when Category is not OTHER
+SPDX_EXTERNAL_REF_TYPES = (
+    (SECURITY, "cpe22Type"),
+    (SECURITY, "cpe23Type"),
+    (SECURITY, "advisory"),
+    (SECURITY, "fix"),
+    (SECURITY, "url"),
+    (SECURITY, "swid"),
+    (PACKAGE_MANAGER, "maven-central"),
+    (PACKAGE_MANAGER, "npm"),
+    (PACKAGE_MANAGER, "nuget"),
+    (PACKAGE_MANAGER, "bower"),
+    (PACKAGE_MANAGER, "purl"),
+    (PERSISTENT_ID, "swh"),
+    (PERSISTENT_ID, "gitoid"),
+)
+
+
 class ExternalRef(SPDXEntry):
     """Reference an external source of information relevant to the package.
 
-    See 7.21 External reference field
+    See 7.21 `External reference field
+    <https://spdx.github.io/spdx-spec/v2.3/package-information/#721-external-reference-field>`_
     """
 
-    json_entry_key = "external-refs"
+    json_entry_key = "externalRefs"
 
     def __init__(
         self,
@@ -467,6 +574,8 @@ class ExternalRef(SPDXEntry):
 
 
 class RelationshipType(Enum):
+    """Describes the type of relationship between two SPDX elements."""
+
     #  Is to be used when SPDXRef-DOCUMENT describes SPDXRef-A
     DESCRIBES = auto()
     #  Is to be used when SPDXRef-A is described by SPDXREF-Document
@@ -488,7 +597,7 @@ class RelationshipType(Enum):
     DEV_DEPENDENCY_OF = auto()
     #  Is to be used when SPDXRef-A is an optional dependency of SPDXRef-B
     OPTIONAL_DEPENDENCY_OF = auto()
-    #  Is to be used when SPDXRef-A is a to be provided dependency of SPDXRef-B
+    #  Is to be used when SPDXRef-A is to be a provided dependency of SPDXRef-B
     PROVIDED_DEPENDENCY_OF = auto()
     #  Is to be used when SPDXRef-A is a test dependency of SPDXRef-B
     TEST_DEPENDENCY_OF = auto()
@@ -574,7 +683,8 @@ class RelationshipType(Enum):
 class Relationship(SPDXEntry):
     """Provides information about the relationship between two SPDX elements.
 
-    See 11.1 Relationship field
+    See 11.1 `Relationship field
+    <https://spdx.github.io/spdx-spec/v2.3/relationships-between-SPDX-elements/#111-relationship-field>`_.
     """
 
     def __init__(
@@ -585,11 +695,11 @@ class Relationship(SPDXEntry):
     ) -> None:
         """Initialize a Relationship object.
 
-        :param left: the left side of the relationship, should be the SPDXID
-            of an element
+        :param spdx_element_id: the left side of the relationship, should be the
+            SPDXID of an element
         :param relationship_type: the type of the relationship
-        :param right: the right side of the relationship, should be the SPDXID
-            of an element
+        :param related_spdx_element: the right side of the relationship, should
+            be the SPDXID of an element
         """
         self.spdx_element_id = spdx_element_id
         self.relationship_type = relationship_type
@@ -611,7 +721,95 @@ class Relationship(SPDXEntry):
 
 @dataclass
 class Package(SPDXSection):
-    """Describe a package."""
+    """Describe a package.
+
+    If the SPDX information describes a package, the following fields shall be
+    included per package. See `7 Package information section
+    <https://spdx.github.io/spdx-spec/v2.3/package-information/>`_
+
+    :ivar PackageName name: A mandatory single line of text identifying the full
+        name of the package as given by the Package Originator
+        (:class:`PackageOriginator`).
+    :ivar SPDXID spdx_id: Uniquely identify any element in an SPDX document
+        which may be referenced by other elements. These may be referenced
+        internally and externally with the addition of the SPDX document
+        identifier. Generally made of ``f"{name}-{version}"``.
+    :ivar PackageVersion version: Identify the version of the package.
+    :ivar PackageFileName file_name: Provide the actual file name of the
+        package, or path of the directory being treated as a package. This may
+        include the packaging and compression methods used as part of the file
+        name, if appropriate.
+    :ivar list[PackageChecksum] checksum: Provide an independently reproducible
+        mechanism that permits unique identification of a specific package that
+        correlates to the data in this SPDX document. This identifier enables a
+        recipient to determine if any file in the original package has been
+        changed. If the SPDX document is to be included in a package, this value
+        should not be calculated. The SHA1 algorithm shall be used to provide
+        the checksum by default. The only supported checksum algorithms (for
+        now) are :class:`SHA1` and :class:`SHA256`.
+    :ivar PackageSupplier supplier: Identify the actual distribution source for
+        the package/directory identified in the SPDX document. This might or
+        might not be different from the originating distribution source for the
+        package. The name of the Package Supplier shall be an organization or
+        recognized author and not a website. For example, SourceForge is a host
+        website, not a supplier, the supplier for
+        https://sourceforge.net/projects/bridge/ is *The Linux Foundation*.
+    :ivar PackageOriginator originator: If the package identified in the SPDX
+        document originated from a different person or organization than
+        identified as Package Supplier (see *supplier* above), this field
+        identifies from where or whom the package originally came. In some
+        cases, a package may be created and originally distributed by a
+        different third party than the Package Supplier of the package. For
+        example, the SPDX document identifies the package as ``glibc`` and the
+        Package Supplier as *Red Hat*, but the *Free Software Foundation* is the
+        Package Originator.
+    :ivar PackageCopyrightText copyright_text: Identify the copyright holders of
+        the package, as well as any dates present. This will be a free form text
+        field extracted from package information files.
+    :ivar FilesAnalyzed files_analyzed: Indicates whether the file content of
+        this package has been available for or subjected to analysis when
+        creating the SPDX document. If false, indicates packages that represent
+        metadata or URI references to a project, product, artifact, distribution
+        or a component. If ``False``, the package shall not contain any files.
+    :ivar PackageLicenseConcluded license_concluded: Contain the license the
+        SPDX document creator has concluded as governing the package or
+        alternative values, if the governing license cannot be determined.
+    :ivar PackageLicenseComments | None license_comments: This field provides a
+        place for the SPDX document creator to record any relevant background
+        information or analysis that went in to arriving at the Concluded
+        License for a package. If the Concluded License does not match the
+        Declared License or License Information from Files, this should be
+        explained by the SPDX document creator. It is also preferable to include
+        an explanation here when the Concluded License is :attr:`NOASSERTION`.
+    :ivar PackageLicenseDeclared license_declared: List the licenses that have
+        been declared by the authors of the package. Any license information
+        that does not originate from the package authors, e.g. license
+        information from a third-party repository, should not be included in
+        this field.
+    :ivar PackageHomePage | None homepage: Provide a place for the SPDX document
+        creator to record a website that serves as the package's home page. This
+        link can also be used to reference further information about the package
+        referenced by the SPDX document creator.
+    :ivar PackageDownloadLocation download_location: This section identifies the
+        download Uniform Resource Locator (URL), or a specific location within a
+        version control system (VCS) for the package at the time that the SPDX
+        document was created.
+    :ivar list[ExternalRef] | None external_refs: An External Reference allows a
+        Package to reference an external source of additional information,
+        metadata, enumerations, asset identifiers, or downloadable content
+        believed to be relevant to the Package. For instance:
+
+        .. code-block:: python
+
+                ExternalRef(
+                    reference_category=ExternalRefCategory.package_manager,
+                    reference_type="purl",
+                    reference_locator="pkg:generic/my-dep@1b2"
+                )
+    :ivar PackageComment | None comment: This field provides a place for the
+        SPDX document creator to record any general comments about the package
+        being described.
+    """  # noqa RST304
 
     name: PackageName
     spdx_id: SPDXID
@@ -623,9 +821,12 @@ class Package(SPDXSection):
     copyright_text: PackageCopyrightText
     files_analyzed: FilesAnalyzed
     license_concluded: PackageLicenseConcluded
+    license_comments: PackageLicenseComments | None
     license_declared: PackageLicenseDeclared | None
+    homepage: PackageHomePage | None
     download_location: PackageDownloadLocation
     external_refs: list[ExternalRef] | None
+    comment: PackageComment | None = field(default=None)
 
 
 @dataclass
@@ -634,7 +835,7 @@ class DocumentInformation(SPDXSection):
 
     document_name: DocumentName
     document_namespace: DocumentNamespace = field(init=False)
-    version: SPDXVersion = SPDXVersion("SPDX-1.2")
+    version: SPDXVersion = SPDXVersion("SPDX-2.3")
     data_license: DataLicense = DataLicense("CC0-1.0")
     spdx_id: SPDXID = SPDXID("DOCUMENT")
 
@@ -644,7 +845,7 @@ class DocumentInformation(SPDXSection):
 
 @dataclass
 class CreationInformation(SPDXSection):
-    """Document where and by who the SPDX document has been created."""
+    """Document where and by whom the SPDX document has been created."""
 
     creators: list[Creator]
     created_now: Created = field(init=False)
@@ -666,8 +867,9 @@ class Document:
     ) -> None:
         """Initialize the SPDX Document.
 
-        :param doc_info: A DocumentInformation instance
-        :param creation_info: A CreationInformation instance
+        :param document_name: The name of this document.
+        :param creators: A list of Entity objects, considered as the creators
+            of this document.
         """
         self.doc_info = DocumentInformation(document_name=DocumentName(document_name))
         self.creation_info = CreationInformation(
@@ -684,44 +886,14 @@ class Document:
 
     def add_package(
         self,
-        name: str,
-        version: str,
-        file_name: str,
-        checksum: list[PackageChecksum],
-        license_concluded: str,
-        supplier: Entity | Literal["NOASSERTION"],
-        originator: Entity | Literal["NOASSERTION"],
-        download_location: str,
-        files_analyzed: bool,
-        copyright_text: str,
-        license_declared: str | None = None,
+        package: Package,
         is_main_package: bool = False,
         add_relationship: bool = True,
-        external_refs: list[ExternalRef] | None = None,
     ) -> SPDXID:
         """Add a new Package and describe its relationship to other elements.
 
-        :param name: the full name of the package
-        :param version: the package version
-        :param file_name: the actual file name of the package
-        :param checksum: the package checksum (see SHA1, SHA256 classes)
-        :param license_concluded: the license concluded as govering the package
-        :param license_declared: the license declared in the package
-        :param supplier: actual distribution source for the package
-        :param originator: this field identifies from where or whom the package
-            originally came
-        :param download_location: download URL for the package at the time that
-            the SPDX document was created
-        :param files_analyzed: whether the file content of this package has
-            been available for or subjected to analysis when creating the
+        :param package: An already created :class:`Package` to be added to this
             SPDX document
-        :param copyright_text: identify the copyright holders of the package,
-            as well as any dates present
-        :param relationship_spdx_element_id: the element SPDXID with which the
-            package has a relationship, note that the relationship will be written as
-            <related_elemnent> <relationship_type> <the new package>
-        :param relationship_type: describe the relation of this package to
-            the related element
         :param is_main_package: whether the package is the main package, in
             which case a relationship will automatically be added to record
             that the document DESCRIBES this package. If false, it is assumed
@@ -730,52 +902,29 @@ class Document:
         :param add_relationship: whether to automatically add a relationship
             element - either (DOCUMENT DESCRIBES <main package>) if is_main_package
             is True or (<main package> CONTAINS <package>)
-        :param external_refs: list of ExternalRef to document
 
         :return: the package SPDX_ID
-        """
-        if not name.endswith(version):
-            new_package_spdx_id = f"{name}-{version}"
-        else:
-            new_package_spdx_id = name
-
-        if is_main_package:
+        """  # noqa RST304
+        if is_main_package and not package.spdx_id.value.endswith("-pkg"):
             # This is the main package, given that is often occurs that
             # a main package depends on a source package of the same name
             # appends a "-pkg" suffix
-            new_package_spdx_id += "-pkg"
+            package.spdx_id = SPDXID(f"{package.spdx_id.value}-pkg")
 
-        new_package = Package(
-            name=PackageName(name),
-            spdx_id=SPDXID(new_package_spdx_id),
-            version=PackageVersion(version),
-            file_name=PackageFileName(file_name),
-            checksum=checksum,
-            license_concluded=PackageLicenseConcluded(license_concluded),
-            license_declared=PackageLicenseDeclared(license_declared)
-            if license_declared is not None
-            else None,
-            supplier=PackageSupplier(supplier),
-            originator=PackageOriginator(originator),
-            download_location=PackageDownloadLocation(download_location),
-            files_analyzed=FilesAnalyzed(files_analyzed),
-            copyright_text=PackageCopyrightText(copyright_text),
-            external_refs=external_refs,
-        )
-        if new_package.spdx_id in self.packages:
+        if package.spdx_id in self.packages:
             raise InvalidSPDX(
-                f"A package with the same SPDXID {new_package.spdx_id}"
+                f"A package with the same SPDXID {package.spdx_id}"
                 " has already been added"
             )
         if is_main_package:
-            self.main_package_spdx_id = new_package.spdx_id
+            self.main_package_spdx_id = package.spdx_id
 
         if add_relationship:
             if is_main_package:
                 relationship = Relationship(
                     spdx_element_id=self.spdx_id,
                     relationship_type=RelationshipType.DESCRIBES,
-                    related_spdx_element=new_package.spdx_id,
+                    related_spdx_element=package.spdx_id,
                 )
             else:
                 if self.main_package_spdx_id is None:
@@ -783,12 +932,12 @@ class Document:
                 relationship = Relationship(
                     spdx_element_id=self.main_package_spdx_id,
                     relationship_type=RelationshipType.CONTAINS,
-                    related_spdx_element=new_package.spdx_id,
+                    related_spdx_element=package.spdx_id,
                 )
 
             self.relationships.append(relationship)
-        self.packages[new_package.spdx_id] = new_package
-        return new_package.spdx_id
+        self.packages[package.spdx_id] = package
+        return package.spdx_id
 
     def add_relationship(self, relationship: Relationship) -> None:
         """Add a new relationship to the document.
@@ -816,16 +965,41 @@ class Document:
         add_section("Creation Info")
         output += self.creation_info.to_tagvalue()
 
+        # Keep track of the packages which should appear first.
+        pkg_ids: list[SPDXID] = []
         add_section("Relationships")
-        for rel in self.relationships:
-            output.append(rel.to_tagvalue())
-        for pkg in self.packages.values():
-            add_section("Package")
-            output += pkg.to_tagvalue()
+        relas: list[str] = []
+        rel_ix: int = 0
+        for rel in sorted(
+            self.relationships, key=lambda rela: rela.related_spdx_element
+        ):
+            if (
+                rel.spdx_element_id == self.spdx_id
+                and rel.relationship_type == RelationshipType.DESCRIBES
+            ):
+                # Make the described packages appear first in list.
+                relas.insert(rel_ix, rel.to_tagvalue())
+                rel_ix += 1
+                # Keep track of those leading packages IDs.
+                pkg_ids.append(rel.related_spdx_element)
+            else:
+                relas.append(rel.to_tagvalue())
+
+        output += relas
+
+        packages: list[str] = ["", ""]
+
+        for pkg in sorted(self.packages.values(), key=lambda package: package.name):
+            if pkg.spdx_id in pkg_ids:
+                packages = ["", "", "# Package", ""] + pkg.to_tagvalue() + packages
+            else:
+                packages += ["# Package", ""] + pkg.to_tagvalue() + ["", ""]
+
+        output += packages
         return output
 
     def to_json_dict(self) -> dict[str, Any]:
-        """Generate a representation of a SPDX following the JSON schema.
+        """Generate a representation of an SPDX following the JSON schema.
 
         Generate a dictionary that can be dumped into a JSON.
         """
@@ -833,7 +1007,9 @@ class Document:
         output["creationInfo"] = self.creation_info.to_json_dict()
         output["documentDescribes"] = []
         output["relationships"] = []
-        for rel in self.relationships:
+        for rel in sorted(
+            self.relationships, key=lambda rela: rela.related_spdx_element
+        ):
             if (
                 rel.spdx_element_id == self.spdx_id
                 and rel.relationship_type == RelationshipType.DESCRIBES
@@ -842,6 +1018,18 @@ class Document:
             else:
                 output["relationships"].append(rel.to_json_dict())
 
-        output["packages"] = [p.to_json_dict() for p in self.packages.values()]
+        packages: list[dict] = []
+        described_ix: int = 0
+        for p in sorted(self.packages.values(), key=lambda package: package.name):
+            if str(p.spdx_id) in output["documentDescribes"]:
+                # Make the described packages appear first in list.
+                packages.insert(described_ix, p.to_json_dict())
+                described_ix += 1
+            else:
+                # Other packages are already sorted by name, append them as
+                # they come.
+                packages.append(p.to_json_dict())
+
+        output["packages"] = packages
 
         return output
