@@ -7,6 +7,8 @@ import textwrap
 import time
 import signal
 
+from pathlib import Path
+
 import e3.fs
 import e3.os.fs
 import e3.os.process
@@ -102,7 +104,7 @@ def test_rlimit():
                 "print('hello'); import sys; sys.stdout.flush(); "
                 "import time; time.sleep(10); print('world')",
             ],
-            timeout=1,
+            timeout=3,
         )
         assert "hello" in p.out
         assert "world" not in p.out
@@ -300,10 +302,10 @@ def test_wait_for_processes():
         bg=True,
     )
     for _ in range(2):
-        result = e3.os.process.wait_for_processes(process_list, 2)
+        result = e3.os.process.wait_for_processes(process_list, 4)
         if result is not None:
             del process_list[result]
-    assert len(process_list) == 0
+    assert len(process_list) == 0, [p.__dict__ for p in process_list]
 
     assert p1.status == 0
     assert p1.out.strip() == "process1"
@@ -332,6 +334,13 @@ def test_run_pipe():
     assert p.status == 0
     assert p.out.strip() == "dummies"
 
+    with open("bunny", "w") as f:
+        f.write("bunny")
+    # Ensure we have support for Path
+    p = e3.os.process.Run(cmd_right, input=Path("bunny"))
+    assert p.status == 0
+    assert p.out.strip() == "bunnies"
+
 
 def test_command_line_image():
     result = e3.os.process.command_line_image(["echo", ""])
@@ -346,7 +355,7 @@ def test_poll():
     )
 
     assert result.poll() is None
-    time.sleep(2)
+    time.sleep(5)
     assert result.poll() == 0
     assert result.out.strip() == "process"
 
